#ifndef ROBUTILS_H
#define ROBUTILS_H

#include "linutil.h"

#define NQMAX 10

typedef double Vec3[3] ;
typedef double Mat3[9] ;
typedef double Por[12] ;

typedef struct
{
	int nq;
	int *ri;
	Vec3 *ui;
	Vec3 *li;
} UnRobot ;

/* Liberation allocations internes structure robot */
void freeRobot(UnRobot *rob);

/*-------------------------------------------------------------
	Lectures paramtres gomtriques robot, a savoir :
	type d'axe, composantes Li-1i, azi et sit Ui.
	et allocation des structures internes de stockage
--------------------------------------------------------------*/
int LitRobot(char *nomrob, UnRobot *Rob);

/*------------------------------------------------------------------
      Calcul modle gomtrique Dop et jacobienne J dans Rp 
          du robot rob, pour la configuration qi           
--------------------------------------------------------------------*/
void qi2DopetJp(UnRobot *rob, double *qi, double *Dop, double *Jacp);

/*----------------------------------------------------------------------
     Calcul configuration qi fournissant la situation Dop
	    dans la mme nappe que le qi initial
	 Renvoi :
	 = nombre d'itrations >= 0 si convergence
	 = -1 si divergence
	 = -2 si nombre max d'itrations atteint
	 = -3 si singularit rsolution
------------------------------------------------------------------------*/
int Dop2qi(UnRobot *rob, double *Dop, double *qi, int maxiter,
  double epsang, double epspos, double dalim, double dplim, double *dqlim);


/************************************************************************/
#if 0 /* POUR TESTER */
/* Donnes  mettre dans le fichier "datarob.txt"                           
1 0 0 1.5  0 90 
1 0 0 0    0  0 
1 0 1 0    0  0 
1 0 1 0   90  0
1 0 0 0    0  0
1 0 0 0   90  0
*/
void main(void)
{
UnRobot *rob = NULL ;
double qi[6] = {-M_PI/2. , M_PI/3. , -2.*M_PI/3. , 0. , M_PI/3. , 0. };
double bid[6];
Por Dop;
double Jacp[36] ;
int i, maxiter = 100;
double epsang = 0.001, epspos = 0.001;
double dalim = 0.2, dplim = 0.2 ;

	i = LitRobot("datarob.txt", &rob) ;
	if (i != 6) 
	{ puts("Erreur lecture donnees Robot"); return;}

	qi2DopetJp(rob, qi, Dop, Jacp) ;
	printpor("Position - Orientation :", Dop);
	printlico("Jacobienne :", Jacp,6,6);

	for (i=0;i<6;i++) qi[i] += 0.05;

	i = Dop2qi(rob, Dop, qi, maxiter, epsang, epspos, dalim, dplim);

	if(i == -1) puts("Divergence de l'algorithme");
	else if(i == -2) puts("Nombre max d'iterations atteint");
	else if(i == -3) puts("Singularite resolution");
	else if (i >= 0) 
	{
		printf("Converge au bout de %d iterations\n",i);
		printlico("qi trouves :", scavec(bid,180./M_PI,qi,6),1,6);
	}

	freeRobot(rob) ;
}
#endif /* POUR TESTER */
/************************************************************************/

#endif /* ROBUTILS_H */
